#include "../include/Spheres.h"
#include <glad/glad.h>
#include <cmath>
#include <glm.hpp>
#include <gtc/type_ptr.hpp>

void generateSphere(std::vector<float>& vertices, std::vector<unsigned int>& indices, float radius, unsigned int sectorCount, unsigned int stackCount)
{
    float x, y, z, xy;
    float nx, ny, nz, lengthInv = 1.0f / radius;

    float sectorStep = 2 * glm::pi<float>() / sectorCount;
    float stackStep = glm::pi<float>() / stackCount;
    float sectorAngle, stackAngle;

    for(unsigned int i = 0; i <= stackCount; ++i)
    {
        stackAngle = glm::pi<float>() / 2 - i * stackStep;
        xy = radius * std::cos(stackAngle);
        z = radius * std::sin(stackAngle);

        for(unsigned int j = 0; j <= sectorCount; ++j)
        {
            sectorAngle = j * sectorStep;
            x = xy * std::cos(sectorAngle);
            y = xy * std::sin(sectorAngle);

            nx = x * lengthInv;
            ny = y * lengthInv;
            nz = z * lengthInv;

            vertices.push_back(x);
            vertices.push_back(y);
            vertices.push_back(z);

            vertices.push_back(nx);
            vertices.push_back(ny);
            vertices.push_back(nz);
        }
    }

    unsigned int k1, k2;
    for(unsigned int i = 0; i < stackCount; ++i)
    {
        k1 = i * (sectorCount + 1);
        k2 = k1 + sectorCount + 1;

        for(unsigned int j = 0; j < sectorCount; ++j, ++k1, ++k2)
        {
            if(i != 0)
            {
                indices.push_back(k1);
                indices.push_back(k2);
                indices.push_back(k1 + 1);
            }

            if(i != (stackCount-1))
            {
                indices.push_back(k1 + 1);
                indices.push_back(k2);
                indices.push_back(k2 + 1);
            }
        }
    }
}

void updateLines(const std::map<std::tuple<int,int,int>, Sphere>& spheres,
                 std::vector<float>& lineVertices, unsigned int lineVBO)
{
    lineVertices.clear();

    for (const auto& [key, sphere] : spheres)
    {
        int x = std::get<0>(key);
        int y = std::get<1>(key);
        int z = std::get<2>(key);

        for (int dx = -1; dx <= 1; ++dx)
        {
            for (int dy = -1; dy <= 1; ++dy)
            {
                for (int dz = -1; dz <= 1; ++dz)
                {
                    if ((std::abs(dx) + std::abs(dy) + std::abs(dz)) != 1)
                        continue;

                    int nx = x + dx;
                    int ny = y + dy;
                    int nz = z + dz;

                    auto neighborKey = std::make_tuple(nx, ny, nz);
                    if (spheres.find(neighborKey) != spheres.end())
                    {
                        if (key < neighborKey)
                        {
                            glm::vec3 neighborPos = spheres.at(neighborKey).position;
                            lineVertices.insert(lineVertices.end(), {
                                sphere.position.x, sphere.position.y, sphere.position.z, 0.5f, 0.5f, 0.5f,
                                neighborPos.x, neighborPos.y, neighborPos.z, 0.5f, 0.5f, 0.5f
                            });
                        }
                    }
                }
            }
        }
    }

    glBindBuffer(GL_ARRAY_BUFFER, lineVBO);
    glBufferData(GL_ARRAY_BUFFER, lineVertices.size() * sizeof(float), &lineVertices[0], GL_DYNAMIC_DRAW);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
}
