#include "../include/UI.h"
#include "../include/HPModelParser.h"
#include "../include/Spheres.h"
#include "../include/Globals.h"
#include "imgui.h"
#include <GLFW/glfw3.h>
#include <string>

void renderUI(GLFWwindow* window, std::map<std::tuple<int,int,int>, Sphere>& spheres,
              std::vector<float>& lineVertices, unsigned int lineVBO)
{
    ImGui::SetNextWindowPos(ImVec2(0, 0));
    ImGui::SetNextWindowSize(ImVec2((float)SIDEBAR_WIDTH, (float)SCR_HEIGHT));
    ImGui::Begin("Control Panel", NULL, ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoMove | ImGuiWindowFlags_NoCollapse);

    static int spherePos[3] = {0, 0, 0};
    ImGui::InputInt("X Position", &spherePos[0]);
    ImGui::InputInt("Y Position", &spherePos[1]);
    ImGui::InputInt("Z Position", &spherePos[2]);

    static bool highlight = false;
    ImGui::Checkbox("Hydrophobic", &highlight);

    if (ImGui::Button("Add Sphere"))
    {
        Sphere sphere;
        sphere.position = glm::vec3(spherePos[0], spherePos[1], -spherePos[2]);
        sphere.color = highlight ? glm::vec3(1.0f,0.0f,0.0f) : glm::vec3(0.5f,0.5f,0.5f);

        spheres[std::make_tuple(spherePos[0],spherePos[1],spherePos[2])] = sphere;
        updateLines(spheres, lineVertices, lineVBO);
    }

    ImGui::Checkbox("Show Grid Points", &showGridPoints);

    ImGui::InputInt("Sphere Size", &sphereSizeInt);
    if (sphereSizeInt < 1) sphereSizeInt = 1;
    if (sphereSizeInt > 10) sphereSizeInt = 10;
    sphereScale = (float)sphereSizeInt;

    ImGui::InputInt("Line Thickness", &lineThicknessInt);
    if (lineThicknessInt < 1) lineThicknessInt = 1;
    if (lineThicknessInt > 10) lineThicknessInt = 10;
    lineThickness = (float)lineThicknessInt;

    ImGui::Text("JSON Input:");
    ImGui::InputTextMultiline("##jsonInput", jsonInput, sizeof(jsonInput), ImVec2(-1.0f, 150.0f));

    if (ImGui::Button("Submit JSON"))
    {
        jsonErrorMessage.clear();
        std::string jsonString(jsonInput);
        if (parseHPModelJson(jsonString, spheres, jsonErrorMessage))
        {
            updateLines(spheres, lineVertices, lineVBO);
        }
    }

    if (!jsonErrorMessage.empty())
    {
        ImGui::TextColored(ImVec4(1.0f,0.0f,0.0f,1.0f), "%s", jsonErrorMessage.c_str());
    }

    ImGui::End();
}
